@@ -420,7 +420,7 @@ func runGC() (freeBytes uintptr) {
 
 	// Mark phase: mark all reachable objects, recursively.
 	markStack()
-	markGlobals()
+	findGlobals(markRoots)
 
 	if baremetal && hasScheduler {
 		// Channel operations in interrupts may move task pointers around while we are marking.
