@@ -1,4 +1,4 @@
-// +build wasm,!wasi
+//go:build wasm && !wasi && !wasip1
 
 package runtime
 
@@ -6,13 +6,19 @@ import "unsafe"
 
 type timeUnit float64 // time in milliseconds, just like Date.now() in JavaScript
 
+// wasmNested is used to detect scheduler nesting (WASM calls into JS calls back into WASM).
+// When this happens, we need to use a reduced version of the scheduler.
+var wasmNested bool
+
 //export _start
 func _start() {
 	// These need to be initialized early so that the heap can be initialized.
 	heapStart = uintptr(unsafe.Pointer(&heapStartSymbol))
 	heapEnd = uintptr(wasm_memory_size(0) * wasmPageSize)
 
+	wasmNested = true
 	run()
+	wasmNested = false
 }
 
 var handleEvent func()
@@ -22,21 +28,6 @@ func setEventHandler(fn func()) {
 	handleEvent = fn
 }
 
-//export resume
-func resume() {
-	go func() {
-		handleEvent()
-	}()
-	scheduler()
-}
-
-//export go_scheduler
-func go_scheduler() {
-	scheduler()
-}
-
-const asyncScheduler = true
-
 func ticksToNanoseconds(ticks timeUnit) int64 {
 	// The JavaScript API works in float64 milliseconds, so convert to
 	// nanoseconds first before converting to a timeUnit (which is a float64),
@@ -52,8 +43,9 @@ func nanosecondsToTicks(ns int64) timeUnit {
 
 // This function is called by the scheduler.
 // Schedule a call to runtime.scheduler, do not actually sleep.
-//export runtime.sleepTicks
+//
+//go:wasmimport gojs runtime.sleepTicks
 func sleepTicks(d timeUnit)
 
-//export runtime.ticks
+//go:wasmimport gojs runtime.ticks
 func ticks() timeUnit
